package leila.algorithm;
import java.io.Serializable;

import javatools.administrative.D;
import javatools.parsers.Char;
import javatools.parsers.Name;
import javatools.parsers.NumberFormatter;
import javatools.parsers.PlingStemmer;
import leila.patterns.LinkageWord;

/**
This class is part of <A HREF=http://www.mpii.mpg.de/~suchanek/downloads/leila target=_blank>LEILA</A>.
The author
<A HREF=http://www.mpi-inf.mpg.de/~suchanek target=_blank>
        Fabian M. Suchanek
</A> reserves all rights to the class. You may use this class
if (1) it is not for commercial purposes, (2) you give credit to the author
and (3) you use the class at your own risk. If you use the class for scientific
purposes, please cite our paper
"Combining Linguistic and Statistical Analysis to Extract Relations from Web Documents"
(<A HREF=http://www.mpi-inf.mpg.de/~suchanek/publications/kdd2006.pdf target=_blank>pdf</A>,
<A HREF=http://www.mpi-inf.mpg.de/~suchanek/publications/kdd2006.ppt target=_blank>ppt</A>,
<A HREF=http://www.mpi-inf.mpg.de/~suchanek/publications/kdd2006.bib target=_blank>bib</A>).
<P>

Pair.java stores a pair of words.
 */
public class Pair implements Serializable, Comparable {
  /** Holds the first word */
  public String word1;
  /** Holds the second word */
  public String word2;
  /** Holds the confidence that this pair is a valid pair */
  public double confidence;
  /** Holds comment */
  public String comment;

  /** Holds the formatter */
  public static NumberFormatter formy=new NumberFormatter("###.##########");

  /** Returns true, if every component of the shorter String appears
   * as a component in the longer one
   */
  public static boolean strequal(String s1, String s2) {
    if(s1.length()>s2.length()) {
      String s3=s1;
      s1=s2;
      s2=s3;
    }
    int j;
    components:
    for(int i=0;i<s1.length();i=j+1) {
      j=s1.indexOf('_',i+1);
      if(j==-1) j=s1.length();
      int found=-1;
      while(-1!=(found=s2.indexOf(s1.substring(i,j),found+1))) {
        if(found>0 && s2.charAt(found-1)!='_') continue;
        if(found+j-i<s2.length() && s2.charAt(found+j-i)!='_') continue;
        continue components;
      }
      return(false);
    }
    return(true);
  }

  /** Compares by confidence */
  public int compareTo(Object o) {
    Pair p=(Pair)o;
    return (p.confidence>this.confidence ? 1 : p.confidence<this.confidence
          ? -1 : 0);
  }

  /** Returns the sum of the word hashcodes */
  public int hashCode() {
    return (word1.hashCode()+word2.hashCode());
  }

  /** Creates a new pair from two words, a confidence and a comment */
  public Pair(String f, String s, double c, String fi) {
    if(s.startsWith("@")) s=s.substring(1);
    if(s.endsWith("@")) s=s.substring(0,s.length()-1);
    f=Char.decodeAndNormalize(f.trim()).replace(' ', '_').replaceAll("[,']","").replaceAll("([^_])&_","$1_&_").replace('-', '_');
    if(!Name.isName(s)) s=PlingStemmer.stem(s);
    if(Name.isCompanyName(f)) f=new Name.CompanyName(f).normalize();
    f=f.toLowerCase();
    s=Char.decodeAndNormalize(s.trim().replace(' ', '_').toLowerCase()).replaceAll("[,']","").replaceAll("([^_])&_","$1_&_").replace('-', '_');
    if(f.endsWith(".") && f.length()>1) f=f.substring(0,f.length()-1);
    if(s.endsWith(".") && s.length()>1) s=s.substring(0,s.length()-1);
    if(f.startsWith("a_")) f=f.substring(2);
    if(s.startsWith("a_")) s=s.substring(2);
    word1=f;
    word2=s;
    confidence=c;
    comment=fi;
  }

  /** Creates a new pair from two words, a confidence and a comment */
  public Pair(LinkageWord w1, LinkageWord w2, double c, String com) {
    this(w1.word,w2.word,c,com);
  }
  /** Creates a new pair from two words and a confidence */
  public Pair(LinkageWord w1, LinkageWord w2, double c) {
    this(w1,w2,c,null);
  }
  /** Creates a new pair from two words*/
  public Pair(String w1,String w2) {
    this(w1,w2,1.0,null);
  }

  /** Tells if both words are strequal */
  public boolean equals(Object o) {
    if(!(o instanceof Pair)) return(false);
    Pair p=((Pair) o);
    return(strequal(this.word1,p.word1) && strequal(this.word2,p.word2));
  }
  /** Returns "<confidence> <word1> <word2> [# comment]" */
  public String toString() {
    return(formy.format(confidence)+" "+word1+' '+word2+(comment==null?"":" # "+comment));
  }

  /** Test routine */
  public static void main(String[] args) {
    while(true) {
      D.p(strequal(D.r(),D.r()));
    }
  }
}