package leila.classifiers;
import java.io.OutputStreamWriter;
import java.io.Serializable;
import java.io.StringWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import leila.patterns.Bridge;
import leila.patterns.LinkagePattern;
import leila.patterns.StoredBridge;
/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).
<P>

Classifier.java represents a Classifier for a set of FeatureObjects.
Predefined Classifiers are KNNClassifier and SVMClassifier.
*/

public abstract class Classifier implements Serializable {
  /** Tells whether contract() has been called */
  protected boolean isContracted=false;
  /** Contains number of featureObjects */
  protected int numFeatureObjects=0;
  /** Maps all distinct bridges to their StoredBridge*/
  public Map<Bridge,StoredBridge> bridgeMap=new HashMap<Bridge,StoredBridge>();

  /** Adds a new LinkagePattern */
  public abstract void add(LinkagePattern o) throws Exception;

  /** Returns the list of featureObjects, might be long, so use printTo instead! */
  public String toString() {
    StringWriter w=new StringWriter();
    try {
      printTo(w);
    } catch(Exception e){};
    return(w.getBuffer().toString());
  }

  /** Prints this classifier */
  public abstract void printTo(Writer o) throws Exception;

  /** Prints this classifier to System.out */
  public void print() throws Exception {
    printTo(new OutputStreamWriter(System.out));
  }

  /** Tells whether the classifier classifies a LinkagePattern as positive */
  public abstract double classify(LinkagePattern o) throws Exception;

  /** Trains the classifier on all added FeatureObjects */
  public abstract void train() throws Exception;

}