package leila.evaluate;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javatools.administrative.D;
import javatools.filehandlers.FileSet;
import leila.parsing.HTMLReader;

/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

Check.java provides a user-interface to verify a LEILA-result manually.
Call this program with the folder containing the corpus files
and the type of the corpus files (html or txt). Then, paste an
output pair and the program will show all parts of the corpus that contain
the ouput pair.
Example:
<PRE>
>java Check c:\data\composers html
Paste "First Second # file":> Chopin composer # chopin.lgo
...
   Chopin was one of the greates composers.
   His works include numerous pieces,
...
</PRE>
*/
public class Check {

  /** Input pattern */
  public static Pattern resultPattern=Pattern.compile("\\d* ?(\\w+) (\\w+) # (\\w+) ?.*");

  /** Reads a result from the keyboard, prints corresponding
   * lines in source document*/
  public static void main(String[] argv) throws Exception {
    if(argv.length<2) {
      System.err.println("\n\n   Check <folder> <extension> \n");
      System.err.println("Allows to check results outputPairs by LEILA.");
      System.err.println("<folder> - The folder of the corpus files");
      System.err.println("<extension> - The extension of the corpus files (html or txt)\n");
      System.err.println("When run, this program accepts chunks of the form");
      System.err.println("      <First> <Second> # <file>");
      System.err.println("It opens <folder>/<file>.<extension>, seeks <First>");
      System.err.println("and prints surrounding lines.");
      System.exit(63);
    }
    String folder=argv[0];
    String ext=argv[1];

    while(true) {
      System.out.print("Paste \"<First> <Second> # <file>\": ");
      String input=D.r();
      if(input==null || input.length()==0) break;
      Matcher m=resultPattern.matcher(input);
      if(!m.matches()) continue;
      String first=m.group(1);
      String second=m.group(2);
      String file=folder+'\\'+FileSet.newExtension(m.group(3),ext);
      System.out.println("File: "+file);
      System.out.println("First:  "+first);
      System.out.println("Second: "+second);
      if(first.indexOf('_')!=-1) first=first.substring(0,first.indexOf('_'));
      if(second.indexOf('_')!=-1) second=second.substring(0,second.indexOf('_'));
      try {
        BufferedReader in=null;
        HTMLReader inhtml=null;
        if(ext.startsWith("htm")) inhtml=new HTMLReader(new FileReader(file));
        else in=new BufferedReader(new FileReader(file));
        String last="";
        String line="";
        while(line!=null) {
          if(line.toLowerCase().indexOf(first)!=-1 || line.toLowerCase().indexOf(second)!=-1) {
            System.out.println("         ---");
            System.out.println("  "+last);
            System.out.println("  "+line);
            last=line;
            if(in!=null) line=in.readLine();
            else line=inhtml.readText(70);
            System.out.println("  "+line);
            continue;
          }
          last=line;
          if(in!=null) line=in.readLine();
          else line=inhtml.readText(70);
        }
        if(in!=null) in.close();
        else inhtml.close();
        System.out.println();
      }catch(IOException e) {
        System.out.println(e+"\n");
      }
    }
  }

}