package leila.parsing;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;

import javatools.administrative.Announce;
import javatools.administrative.Parameters;
import javatools.filehandlers.FileSet;
import leila.patterns.LinkageFile;


/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

LGParse.java provides a wrapper for the Link Grammar-Parser. The following
parameters are required in leila.ini:
<UL>
<LI> parserCommand - the command to evoke the parser with input file %1 and output file %2.<BR>
For example, with Windows, this could look like "cmd /c c:\lg\link41b.exe < %1 > %2"</LI>
<LI> parserFolder - the parser folder</LI>
</UL>
 */
public class LGParse {

   /** Calls the LG-Parser with a LGI-file (call by Java). Returns TRUE in case of success */
  public static boolean main(File f) throws Exception {
      Parameters.ensureParameters(
        "parserCommand - the command to evoke the parser",
        "parserFolder - the parser folder"
      );
      String PARSERCMD=Parameters.get("parserCommand");
      String PARSERFOLDER=Parameters.get("parserFolder");
      f=LinkageFile.lgiFile(f);
      Announce.doing("Parsing "+f.getName());
      PARSERCMD=PARSERCMD.replace("%1", f.getAbsolutePath())
                .replace("%2", LinkageFile.lgoFile(f).getAbsolutePath());
      Process parsy=Runtime.getRuntime().exec(PARSERCMD,null,new File(PARSERFOLDER));
      BufferedReader bri=new BufferedReader(new InputStreamReader(parsy.getInputStream ()));
      BufferedReader bre=new BufferedReader(new InputStreamReader(parsy.getErrorStream ()));
      while(null!=bri.readLine() || null!=bre.readLine()) ; //Loop & ignore
      parsy.waitFor();
      if(parsy.exitValue()==0) {
        Announce.done();
        return(true);
      }
      else {
        Announce.failed();
        return(false);
      }
  }

  /** Calls the LG-Parser with a file (call by User)*/
  public static void main(String[] argv) throws Exception {
    if(argv.length<1) {
      System.err.println("\n\n   LGParse <LGIfile> \n");
      System.err.println("Calls the Link Grammar parser for an LGI-file (Link Grammar Input),");
      System.err.println("producing a corresponding LGO-file (Link Grammar Output).");
      System.err.println("<LGIfile> - file to be parsed,");
      System.err.println("            may contain quotes (enclose the filename by quotes).");
      System.exit(63);
    }
    Parameters.init("leila.ini");
    FileSet files=new FileSet(argv[0]);
    if(!files.iterator().hasNext()) {
      System.err.println("Can't find "+argv[0]);
      return;
    }
    for(File f : files) main(f);
  }
}