package leila.patterns;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

Bridge.java stores a bridge, i.e. a list of Links. The only difference to a List
is the implementation of hashCode() and equals(), which do not take into
account the last word.
This is a Wrapper Class, i.e. no deep copies are made.
*/
public final class Bridge implements Serializable {
  /** Holds the links */
  public List<Link> links;
  /** Constructs a Bridge */
  public Bridge(List<Link> l) {
    links=l;
  }
  /** Empty constructor for constructors of subclasses*/
  private Bridge() {}
  /** Returns a hashCode for this Bridge */
  public int hashCode() {
    int result=0;
    for(int i=0;i<links.size()-1;i++)
      result^=links.get(i).hashCode();
    result^=links.get(links.size()-1).toInt();
    return(result);
  }
  /** Two Bridges are equal if their links are the same (except for the last destination word)*/
  public boolean equals(Object o) {
    if(o==null || !(o instanceof Bridge) || links.size()!=((Bridge)o).links.size()) return(false);
    for(int i=0;i<links.size()-1;i++)
      if(!links.get(i).equals(((Bridge)o).links.get(i))) return(false);
    return(links.get(links.size()-1).wequals(((Bridge)o).links.get(links.size()-1)));
  }
  /** Returns a String representation */
  public String toString() {
    if(links.size()==0) {
      return("<empty bridge>");
    }
    StringBuilder r=new StringBuilder();
    switch(links.get(0).direction) {
      case RIGHT:
        r.append("<first>");
        for(Link l : links) r.append(' ').append(l);
        r.setLength(r.lastIndexOf(" "));
        r.append(" <second>");
        break;
      case LEFT:
        for(int i=links.size()-1;i>=0;i--) r.append(links.get(i)).append(' ');
        r.replace(0,r.indexOf(" "),"<second>");
        r.append("<first>");
        break;
    }
    return(r.toString());
  }
  /** Returns a new Bridge with all Links essentialized */
  public Bridge essentialize() {
    Bridge result=new Bridge();
    result.links=new ArrayList<Link>(this.links.size());
    for(Link l : links)
        result.links.add(l.essentialize());
    return(result);
  }
}