package leila.patterns;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

LinearLinkageFile represents an LL-file (the non-grammatical parts of the corpus).
The class provides an Iterator over the LinearLinkages.
*/

public class LinearLinkageFile implements Iterable<LinearLinkage>, Iterator<LinearLinkage> {
  /** Holds the Reader */
  protected BufferedReader in;
  /** For iteration */
  private String next;

  /** Constructs a LinearLinkageFile from a file */
  public LinearLinkageFile(File inf) throws Exception {
    in=new BufferedReader(new FileReader(llFile(inf)));
    next=in.readLine();
  }

  /** Returns this */
  public Iterator<LinearLinkage> iterator() {
    return(this);
  }

  /** TRUE if there is another Linkage */
  public boolean hasNext() {
    return(next!=null);
  }

  /** Does nothing */
  public void remove() {}

  /** Returns next Linkage */
  public LinearLinkage next() {
    LinearLinkage retme=next==null?null:new LinearLinkage(next);
    try{
      next=in.readLine();
    }catch(IOException chiSeNeFrega) {
      next=null;
    }
    return(retme);
  }

  /** Returns the LL-filename for an HTML-filename */
  public static File llFile(File f) {
    return(new File(f.getPath().substring(0,f.getPath().lastIndexOf('.'))+".ll"));
  }

}