package leila.patterns;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javatools.administrative.D;
import javatools.filehandlers.FileSet;
import leila.parsing.Document;

/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

LinkageFile.java represents an LGO-file (Link Grammar Output).
This class provides an Iterator over the Linkages.
  */
public class LinkageFile implements Iterable<Linkage>, Iterator<Linkage> {
  /** Holds the Reader */
  protected BufferedReader in;
  /** Holds the currect document context */
  protected Document document=new Document();

  /** Constructs a LinkageFile from a file*/
  public LinkageFile(File inf) throws Exception {
    in=new BufferedReader(new FileReader(lgoFile(inf)));
  }

  /** For iterator */
  private String s="";
  /** TRUE if there is another Linkage */
  public boolean hasNext() {
        try {
          while(s!=null && s.indexOf("---")==-1) s=in.readLine();
        } catch(Exception chiSeNeFrega) {
          return(false);
        };
        return(s!=null);
  }

  /** Does nothing */
  public void remove() {}

  /** Returns next Linkage */
  public Linkage next() {
        if(!hasNext()) return(null);
        List<String> l=new ArrayList<String>(15);
        try {
          while(s!=null && !s.equals("")) {
            l.add(s);
            s=in.readLine();
          }
        } catch(Exception wontHappenAnyway) {
          return(null);
        }
        return(new Linkage(l,document));
  }

  /** Returns this */
  public Iterator<Linkage> iterator() {
    return(this);
  }

  /** Returns the LGO-filename for an HTML-filename */
  public static File lgoFile(File f) {
    return(FileSet.newExtension(f,"lgo"));
  }

  /** Returns the LGI-filename for an HTML-filename */
  public static File lgiFile(File f) {
    return(FileSet.newExtension(f,"lgi"));
  }

  /** Test routine */
  public static void main(String[] argv) throws Exception {
    for(Linkage l : new LinkageFile(new File(argv[0]))) D.p(l.linkageWords+" "+l.id);
  }
}