package leila.patterns;
import java.io.Serializable;
import java.util.List;
/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

LinkagePattern.java epresents a pattern. This includes the bridge, the bases and a label.
A LinkagePattern is merely a subgraph of the Linkage.
This is a Wrapper Class, i.e. no deep copies are made.
*/
public class LinkagePattern implements Serializable {

  /** The label (confidence) of this LinkagePattern */
  public double label=1.0;
  /** The two basis words */
  public LinkageWord[] basis=new LinkageWord[2];
  /** The bridge */
  public Bridge bridge;
  /** Identifies the source of this pattern*/
  public int sourceId;

  /** Constructs a LinkagePattern from two basis words and a bridge*/
  public LinkagePattern(LinkageWord f, List<Link> l, LinkageWord s, int sourceId) {
    basis[0]=f;
    basis[1]=s;
    bridge=new Bridge(l);
    this.sourceId=sourceId;
  }

  /** Two FeatureObjects are equal if their bases and bridges coincide */
  public boolean equals(Object o) {
    return(o!=null && o instanceof LinkagePattern &&
           basis[0].equals(((LinkagePattern)o).basis[0]) &&
           basis[1].equals(((LinkagePattern)o).basis[1]) &&
           bridge.equals(((LinkagePattern)o).bridge));
  }

  /** Returns a String representation */
  public String toString() {
    return("LinkagePattern\n"
           +"  Bridge: "+bridge+"\n"
           +"  First: "+basis[0].word+"\n"
           +"  Second: "+basis[1].word+"\n"
           +"  Confidence: "+label+"\n");
  }

  /** returns the hashCode */
  public int hashCode() {
    return(bridge.hashCode());
  }

}