package leila.patterns;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javatools.datatypes.SVMModel;

/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

StoredBridge.java stores a Bridge together with the set of
StoredPatterns that use it. This class uses static variables!
*/

public final class StoredBridge implements Serializable {
  /** Counts all Bridges*/
  public static int bridgeCounter=0;
  /** Identifies this Bridge */
  public int id=bridgeCounter++;
  /** Holds the Bridge itself*/
  public Bridge bridge;
  /** Holds all stored patterns that use this Bridge */
  public List<StoredLinkagePattern> users;
  /** Stores sum of labels of this bridge */
  public double sum=0.0;
  /** Stores the SVMModel for this StoredBridge (used by SVMClassifier) */
  public SVMModel model;

  /** Constructs a StoredBridge, compresses the Bridge's Links */
  public StoredBridge(Bridge b,LinkagePattern f) {
    bridge=b.essentialize();
    users=new ArrayList<StoredLinkagePattern>(1);
    users.add(new StoredLinkagePattern(f));
  }

  /** Returns a hashCode for this Bridge */
  public int hashCode() {
    return(bridge.hashCode());
  }

  /** Two StoredBridges are equal if their bridges are equal*/
  public boolean equals(Object o) {
    return(bridge.equals(o));
  }

  /** Returns a String representation */
  public String toString() {
    return("#"+id+": "+bridge);
  }
}