package leila.patterns;


/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

StoredLinkagePattern represents a LinkagePattern with a null Bridge.
A StoredPattern is clings to a StoredBridge, which holds the pattern's bridge.
*/

public class StoredLinkagePattern extends LinkagePattern {

  /** Contains the number of Feature Objects in this equivalence class */
  public int num=1;

  public StoredLinkagePattern(LinkagePattern f) {
    super(f.basis[0], null, f.basis[1],f.sourceId);
    this.label=f.label;
  }

  /** Two StoredFatureObjects are equal if their bases coincide */
  public boolean equals(Object o) {
    return(o!=null && o instanceof LinkagePattern &&
           basis[0].equals(((LinkagePattern)o).basis[0]) &&
           basis[1].equals(((LinkagePattern)o).basis[1]));
  }

  /** Returns a String representation */
  public String toString() {
    return("StoredLinkagePattern\n"
           +"  First: "+basis[0].word+"\n"
           +"  Second: "+basis[1].word+"\n"
           +"  Confidence "+label+"\n"
           +"  FeatureObjects: "+num+"\n");
  }

  /** Returns a similarity measurement */
  public double similarity(LinkagePattern o) {
    double result=0.0;
    for(int i=0;i<2;i++) {
      //Should be taken care of by the respective relation
      //result+=basis[i].similarity(o.basis[i]);
      double contextSim=0.0;
      for(Link l1 : basis[i].contexts) {
        for(Link l2 : o.basis[i].contexts) {
          contextSim+=l1.similarity(l2);
        }
      }
      result+=contextSim/(basis[i].contexts.size()*o.basis[i].contexts.size());
    }
    return(result);
  }
  /** returns the hashCode */
  public int hashCode() {
    return(basis[0].hashCode()+basis[1].hashCode());
  }

}