package leila.relations;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashMap;
import java.util.Map;

import javatools.administrative.D;
import javatools.administrative.Parameters;
import javatools.parsers.Name;
import leila.patterns.LinkageWord;
import leila.patterns.LinkageWord.Type;


/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).
<P>

Headquarters.java implements a the company/headquarters-relation.
It requires the example pairs in a file of the form
<PRE>
company / headquarters
...
</PRE>
The class requires the following parameter in leila.ini:
<UL>
<LI> headquartersFile = the file with the example pairs
</UL>
 */
public class Headquarters extends Relation {

  /** returns false */
  public boolean requiresCounterexamples() {
     return(false);
  }

  /** Contains the examples */
  protected Map<String,String> samples=new HashMap<String,String>();

  /** Initializes the headquarters relation from the example pair file */
  public Headquarters(String f) throws Exception {
    BufferedReader in=new BufferedReader(new FileReader(f));
    String line;
    while(null!=(line=in.readLine())) {
      String company=line.substring(0,line.indexOf('/')).trim().replace(' ','_').toLowerCase();
      company=Name.of(company).normalize();
      String hq=line.substring(line.indexOf('/')+1).trim().replace(' ','_').toLowerCase();
      samples.put(company,hq);
    }
    in.close();
  }

  /** Creates a relation from the file given by the parameter "headquartersFile" in leila.ini*/
  public Headquarters() throws Exception {
    this(Parameters.get("headquartersFile"));
  }

  /** Tells whether this pair is an example, a counterexample or a candidate */
  public WordPairType assess(LinkageWord w1, LinkageWord w2) {
    if(!w1.is(LinkageWord.Type.NAME)
          || !w2.is(LinkageWord.Type.NAME)) return(WordPairType.NOTHING);
    String hq=samples.get(w1.word.toLowerCase());
    if(hq!=null && hq.equalsIgnoreCase(w2.word)) return(WordPairType.EXAMPLE);
    return(WordPairType.CANDIDATE);
  }

  /** Tells whether this word could be an example */
  public boolean possibleExample(LinkageWord w) {
    return (w.is(LinkageWord.Type.NAME));
  }

  /**Test method*/
  public static void main(String[] args) throws Exception {
    Headquarters h=new Headquarters("c:\\fabian\\ontolearn\\relations\\headquarters.txt");
    while(true) {
      D.p(h.assess(new LinkageWord(D.r()),new LinkageWord(D.r())));
    }
  }

}