package leila.relations;

import leila.patterns.LinkageWord;

/**
 This class is part of LEILA (http://mpii.de/~suchanek/leila).
 It is licensed under the Creative Commons Attribution License 
 (http://creativecommons.org/licenses/by/3.0) 
 by the author Fabian M. Suchanek (http://mpii.de/~suchanek).


Relation.java represents a relation to be learned. To have LEILA run with a new relation,
extend this class and give it as a parameter to Leila.java.
*/

public abstract class Relation {
  public enum WordPairType {EXAMPLE,COUNTEREXAMPLE,CANDIDATE,NOTHING};
  /** Tells whether this relation requires seeking for counterexamples */
  public boolean requiresCounterexamples() {return(false);};
  /** Tells whether a pair of words is an example, a counterexample or a candidate*/
  public WordPairType assess(LinkageWord w1, LinkageWord w2) {
    return(WordPairType.NOTHING);
  }
  /** Tells whether this word could be the first word of an example pair.
   * Whenever a word is the first component of any example pair, this method should
   * return TRUE. If the word is not a such first component, the method can return
   * TRUE or false. Thus, the safest way is to return always TRUE. For efficiency,
   * try to reduce the cases where the method returns TRUE. */
  public boolean possibleExample(LinkageWord w) {
    return(true);
  }
  @Override
  public String toString() {   
    return this.getClass().getSimpleName();
  }
}