package leila.relations;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javatools.administrative.Parameters;
import javatools.filehandlers.FileLines;
import leila.patterns.LinkageWord;
import leila.relations.Relation.WordPairType;
/**
This class is part of LEILA (http://mpii.de/yago-naga/leila).
It is licensed under the Creative Commons Attribution License 
(http://creativecommons.org/licenses/by/3.0) 
by the author Fabian M. Suchanek (http://suchanek.name).

<P>

SimpleFunction.java is a functional relation (i.e. a many-to-one relation),
which reads its examples from a given file. It requires the following parameter
in leila.ini:
<UL>
<LI>SimpleFunction - points to a file containing example pairs
</UL>
By default, the file RelationData/SimpleFunction/examples.txt is read.
The example pairs should be of the format
<PRE>
   first second
   ...
</PRE>
Since the relation is functional, any pair with a first component listed
in the examples and a "wrong" second component is a counterexample. For example,
suppose you store persons with their birth dates in the file. If Chopin is born in 1810,
then Chopin/1811, Chopin/1812, ... are implicitly  counterexamples.
 */
public class SimpleFunction extends Relation {
  /** Holds the examples */
  protected Map<String,String> examples=new TreeMap<String,String>();
  /** Returns true */
  public boolean requiresCounterexamples() {
     return(true);
  }
  /** Tells whether a pair of words is an example, a counterexample or a candidate*/
  public WordPairType assess(LinkageWord w1, LinkageWord w2) {
    // See whether we have some example with the first word as first component
    String m=examples.get(w1.word.toLowerCase());
    // If not so, the pair could be a candidate
    if(m==null) return(WordPairType.CANDIDATE);
    // If so, and if the second component is the one we have in the examples,
    // the pair is an example
    if(m.equalsIgnoreCase(w2.word)) return(WordPairType.EXAMPLE);
    // ...else a counterexample
    else return(WordPairType.COUNTEREXAMPLE);
  }
  /** Tells whether the word appears in the examples */
  public boolean possibleExample(LinkageWord w) {
    return(examples.get(w.word.toLowerCase())!=null);
  }
  
  /** Pattern in the example file */
  public Pattern pattern=Pattern.compile("\\s*(\\S+)\\s+(\\S+)\\s*");
  
  /** Reads the examples from the file given by SimpleRelationExamples */
  public SimpleFunction() throws Exception { 
    for(String s : new FileLines(Parameters.get("SimpleFunction","../RelationData/SimpleFunction/examples.txt"),
          "Parsing SimpleFunction examples")) {
      Matcher m=pattern.matcher(s);
      if(!m.matches()) throw new Exception("The line \n   "+s+"\n"+
            "is not of the expected format\n"+
            "    <letters><space><letters>");
      examples.put(m.group(1).toLowerCase(),m.group(2));
    }
  }
}